/*
 *  Empire - A multi-player, client/server Internet based war game.
 *  Copyright (C) 1986-1998, Dave Pare, Jeff Bailey, Thomas Ruschak,
 *                           Ken Stevens, Steve McClure
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ---
 *
 *  See the "LEGAL", "LICENSE", "CREDITS" and "README" files for all the
 *  related information and legal notices. It is expected that any future
 *  projects/authors will amend these files as needed.
 *
 *  ---
 *
 *  upda.c: Give the time of the next update
 * 
 *  Known contributors to this file:
 *  
 */

#include <stdio.h>
#include <sys/types.h>
#include "misc.h"
#include "player.h"
#include "commands.h"
#include "optlist.h"

#include "keyword.h"
#include "wantupd.h"

extern int update_blitz();
/*
 * Tell what the update policy is, and when the next update
 * is likely to be.
 */
int
upda(void)
{
    if (opt_UPDATESCHED) {
	time_t now, next, delta;
	int pol, wpol;
	int min, absmin;
	s_char *p;

	if (updates_disabled())
		pr("UPDATES ARE DISABLED!\n");

	(void) time(&now);
	demand_update_info(&pol, &wpol, &min, &absmin);
	switch (pol) {
	case UDP_NORMAL:
		next_update_time(&now, &next, &delta);
		pr("\nUpdates occur at times specified by the ETU rates.\n\n");
		pr("The next update is at %19.19s.\n", ctime(&next));
		break;
	case UDP_TIMES:
		next_update_time(&now, &next, &delta);
		pr("\nUpdates occur at scheduled times.\n\n");
		pr("The next update is at %19.19s.\n", ctime(&next));
		break;
        case UDP_BLITZ:
                next_update_time(&now, &next, &delta);
                pr("\nBlitz Updates occur every %d minutes. \n\n",
			player->blitz_time);
                pr("The next update is at %19.19s.\n",ctime(&next));
		break;
	case UDP_NOREG:
		pr("There are no regularly scheduled updates.\n");
		break;
	default:
		pr("Update policy is inconsistent.\n");
	}
	pr("The current time is   %19.19s.\n\n", ctime(&now));

	if (opt_DEMANDUPDATE) {
	    if (wpol != UDDEM_DISABLE) {
		switch (wpol) {
		case UDDEM_TMCHECK:
			next_update_check_time(&now, &next, &delta);
			pr("Demand updates occur at update CHECK times.\n");
			pr("The next update check is at %19.19s.\n",
			   ctime(&next));
			break;
		case UDDEM_COMSET:
			pr("Demand updates occur right after the demand is set.\n");
			break;
		default:
			pr("Update demand policy is inconsistent.\n");
		}
	    }
	}

	if ((pol == UDP_TIMES) ||
	    ((wpol == UDDEM_TMCHECK) && opt_DEMANDUPDATE)) {
		if (NULL != (p=kw_find("update_times")))
			pr("The update schedule is: %s\n", p);
	}
	if (opt_DEMANDUPDATE) {
	    if (wpol != UDDEM_DISABLE) {
		if (NULL != (p=kw_find("update_demandtimes")))
			pr("Demand updates are allowed during: %s\n", p);
		if (min == 0) {
			pr("Demand updates are disabled by a mininum of 0\n");
		} else {
			if (min < absmin) min = absmin;
			pr("Demand updates require %d country(s) to want one.\n",min);
		}
	    }
	}
	if (NULL != (p=kw_find("days")))
		pr("Game days are: %s\n", p);
	if (NULL != (p=kw_find("hours")))
		pr("Game hours are: %s\n", p);

	return(0);
    } else {
	extern	int s_p_etu;
	extern	int etu_per_update;
	extern	int adj_update;
	time_t	now;
	time_t	upd_time;
	time_t	next_update;
	int	secs_per_update;
	int	delta;

	(void) time(&now);
	upd_time = now + adj_update;
	secs_per_update = etu_per_update * s_p_etu;
	delta = secs_per_update - (upd_time % secs_per_update);
	next_update = now + delta;
	pr("The next update is at %19.19s.\n",ctime(&next_update));
	pr("The current time is %19.19s.\n",ctime(&now));
	return 0;
    }
}
